
import argparse
import sys
import pkg_resources
import os

from .api_requester.api_router import ApiRouter
from .todos.todo_router import TodoRouter
from .dir_builder.dir_router import DirectoryRouter

def main():

    try:
        VERSION = pkg_resources.require('assister')[0].version
    except pkg_resources.DistrobutionNotFound:
        VERSION = '1.0.4'

    parser = argparse.ArgumentParser(description='Productivity without a mouse')
    parser.add_argument('command', default=None, action='store', nargs='+')
    parser.add_argument('-p', '--progress', dest='p', default=False, action='store_true')
    parser.add_argument('-g', '--greet', dest='g', default=False, action='store_true')
    args = parser.parse_args()

    command = args.command[0]
    arguments = args.command[1:]

    command_mapper = {
            'todo': TodoRouter,
            'api': ApiRouter,
            'dir': DirectoryRouter,
            }

    if command == 'version':
        sys.stdout.write(f'ASSISTER VERSION | {VERSION}\n')
        exit(0)

    if args.g:
        user = os.getlogin()
        print(f'Hello, {user}')

    r = command_mapper[command]
    i = r(arguments)
    i()





from ..base_models import WriteOut
from .api_service import ApiService

class ApiRouter:
    '''
    assister api <method> <base_url> <body> <headers>
    '''

    def __init__(self, a):
        self.a = a

        if len(self.a) == 2:
            self.service = ApiService(self.a[0], self.a[1])
        elif len(self.a) == 3:
            self.service = ApiService(self.a[0], self.a[1], self.a[2])
        elif len(self.a) == 4:
            self.service = ApiService(self.a[0], self.a[1], self.a[2], self.a[3])


    def __call__(self):
        if self.a is None:
            self.w('Use assister -h for commands', 0)
        self.service()






import os
from .dir_service import DirectoryService


class DirectoryRouter:

    def __init__(self, a: list):
        self.a = a
        self.service = DirectoryService(a)

    def __call__(self):
        self.service()





class Todo:

    def __init__(self, title, content, complete, due):
        self.title = title
        self.content = content
        self.complete = complete
        self.due = due

    def __repr__(self):
        if self.complete:
            status = 'Complete'
        else:
            status = 'Incomplete'
        return self.title, self.content, status, self.due





import os
import time

from tqdm import tqdm


class DirectoryService:

    def __init__(self, args: list):
        self.args = args

    def __call__(self):
        path = os.getcwd()
        if not self.args[0].startswith('/'):
            self.args[0] = f'/{self.args[0]}'
        os.makedirs(f'{path}{self.args[0]}')
        for i in tqdm(self.args[1:]):
            if not i.startswith('/'):
                i = f'/{i}'
            open(path + self.args[0] + i, 'w').close()
            time.sleep(0.1)





import requests
import json
from ..base_models import WriteOut, ReadOut

class ApiService:

    def __init__(self, request_method, url, body=None, headers=None):
        self.w = WriteOut()
        self.r = ReadOut()

        self.request_method = request_method
        self.url = url

        if headers:
            self.headers = headers
        if body:
            self.body = body

    def __call__(self):

        # TODO: Add header Functionality for Post, Put, and Del methods
        method_mapper = {
                    'get': self.get,
                    'post': self.post,
                    'del': self.delete,
                    'put': self.put
                }

        m = method_mapper[self.request_method]
        m()

    def wrap_json(response):
        try:
            r = json.loads(response.text)
            self.w(json.dumps(r, indent=4, sort_keys=True), 0)
        except json.decoder.JSONDecodeError:
            self.w(response.text, 0)

    def get(self):
        if not self.headers:
            response = requests.get(self.url)
        else:
            response = requests.get(self.url, headers=self.headers)
        wrap_json(response)


    def post(self):
        response = requests.post(self.url, data='{}'.format(self.body))
        self.w(str(response), 0)

    def delete(self):
        response = requests.delete(self.url)
        self.w(str(response), 0)

    def put(self):
        response = requests.put(self.url, data='{}'.format(self.body))
        self.w(str(response), 0)





import sys

class WriteOut:

    def __call__(self, msg, code=None):
        sys.stdout.write(msg + '\n')
        if code is 0 or code:
            sys.exit(code)


class ReadOut:

    def __call__(self, msg):
        i = input(msg + '\n>>> ')
        return i





from setuptools import setup

setup(
        name='assister',
        author='Connor Mullett',
        version='1.1.0',
        description='Simple Assister CLI, https://github.com/connormullett/assister',
        packages=['assister/todos', 'assister/api_requester', 'assister/dir_builder'],
        install_requires=['tqdm'],
        include_package_data=True,
        license='MIT',
        url='https://github.com/connormullett/assister',
        entry_points={
            'console_scripts': [
                'ass=assister.__main__:main'
            ]
        }
    )





from ..base_models import WriteOut
from .todo_service import TodoService

class TodoRouter():

    def __init__(self, t):
        self.t = t
        self.service = TodoService()
        self.w = WriteOut()

    def __call__(self):
        if self.t is None:
            self.w('Use assister -h for commands', 0)
        if len(self.t) > 1:
            self.todo_arg_router()
        else:
            self.router(self.t[0])

    def router(self, a):
        function_mapper = {
                    'create': self.service.todo_create,
                    'view': self.service.view_todos,
                }
        try:
            b = function_mapper[a]
            b()
        except Exception as e:
            self.w('An Error Occured, {}'.format(e), 1)

    def todo_arg_router(self):
        function_mapper = {
                    'del': self.service.todo_delete,
                    'mc': self.service.mark_complete,
                    'mi': self.service.mark_incomplete,
                    'cdue': self.service.change_due,
                    'update': self.service.todo_update
                }
        b = function_mapper[self.t[0]]
        b(self.t[1])





import sys
import os
import csv
import pkg_resources
import time

from ..base_models import WriteOut, ReadOut
from .models import Todo
from datetime import datetime


class TodoService:

    def __init__(self):
        path = 'todo.csv'
        self.todo_file = pkg_resources.resource_filename(__name__, path)
        self.r = ReadOut()
        self.w = WriteOut()

    def mark_complete(self, todo_id):
        todos = self.read_todos()
        todo_str = todos[int(todo_id)]
        title, content, complete, due = todo_str.split(',')
        todo = Todo(title, content, bool(complete), due)
        todo.complete = True
        self.w('Todo marked complete')
        self.delete_row(int(todo_id))
        self.write_row(todo)

    def mark_incomplete(self, todo_id):
        todos = self.read_todos()
        todo_str = todos[int(todo_id)]
        title, content, complete, due = todo_str.split(',')
        todo = Todo(title, content, bool(complete), due)
        todo.complete = False
        self.delete_row(int(todo_id))
        print(todo.__repr__())
        self.write_row(todo)

    def todo_create(self):

        while True:
            title = self.r('Enter title (max 20 characters)')
            if len(title) > 20:
                continue
            else:
                break
        while True:
            content = self.r('Enter todo body (max 50 characters)')
            if len(content) > 50:
                continue
            else:
                break

        # TODO: Add handling for dates with errors
        due = self.r('Enter due date (DD/MM/YYYY)')
        t = Todo(title, content, False, due)
        self.write_row(t)

    def write_row(self, t):
        with open(self.todo_file, 'a') as f:
            writer = csv.writer(f, delimiter=',')
            writer.writerow(t.__repr__())


    def read_todos(self):
        with open(self.todo_file, 'r') as f:
            _ = f.readlines()
            todos = []
            del _[0]
            for todo in _:
                todo = todo.rstrip()
                todos.append(todo)
            return todos


    def view_todos(self):
        todos = self.read_todos()
        for i, todo in enumerate(todos):
            t = todo.__repr__().split(',')
            t[0] = t[0].replace("'", '')
            t[-1] = t[-1].replace("'", '')
            self.w(f'{i}\t{t[0]}\t{t[1]}\t\t{t[2]}\t{t[3]}')
        self.w('', 0)

    def todo_delete(self, t):

        todos = self.read_todos()
        i = int(t)

        self.w(todos[i])
        try:
            choice = self.r('are you sure you want to delete this todo? (y/n)')
        except Exception as e:
            self.w('Invalid ID\n', 1)
        if choice.lower() == 'n':
            sys.exit(0)
        elif choice.lower() == 'y':
            self.delete_row(i)

    def change_due(self):
        pass

    def todo_update(self):
        pass

    def delete_row(self, i):
        with open(self.todo_file, 'r') as f:
            rows = f.readlines()
            del rows[i + 1]

        with open(self.todo_file, 'w') as f:
            for row in rows:
                f.write(row)

